#!/usr/bin/perl -w

use Test::More 'no_plan';
use Carp;
use strict;

BEGIN { use_ok('Data::Array2ArrayMap::Hash::MultiKey') }
BEGIN { use_ok('Data::Array2ArrayMap::Hash::Tree') }
BEGIN { use_ok('Data::Array2ArrayMap::Hash::XSTree') }

print "\nSolid testing\n";

my $c=Data::Array2ArrayMap::Hash::MultiKey->new();
test_suite ($c);

print "\nTree testing\n";

  $c=Data::Array2ArrayMap::Hash::Tree->new();
test_suite ($c);

print "\nXSTree testing\n";

  $c=Data::Array2ArrayMap::Hash::XSTree->new();
test_suite ($c);

sub test_suite {
    my $c=shift;
    my @a1=(5,76,8,8,8);
    my @a2=(2,6,89,0,89,0);
    $c->set(["a","b"],\@a1);
    $c->set(["c","d"],\@a2);
    compare("set testing",join(':',$c->get(["a","b"])),join(':',@a1));
    compare("set testing",join(':',$c->get(["c","d"])),join(':',@a2));
   
    assertTrue("exists testing",$c->get_ref_if(["a","b"]));
    assertFalse("exists testing",$c->get_ref_if(["a","c"]));

    $c->delete(["a","b"]);
    $c->delete(["c","d"]);
    
    compare(ref($c)." delete testing",arraypack($c->get(["c","d"])),'');
    compare(ref($c)." delete testing",arraypack($c->get(["a","b"])),'');

    $c->set(["a","b"],\@a1);
    $c->set(["c","d"],\@a2);
    
    $c->delete(["a","b"]);
    $c->delete(["c","d"]);
    my @val0=$c->get(["c","d"]);
    ok(@val0==0, "delete should produce an empty array");

    compare(ref($c)." delete testing",join(':',$c->get(["c","d"])),'');
    compare(ref($c)." delete testing",join(':',$c->get(["a","b"])),'');
    
    $c->set(["a","b"],\@a1);
    $c->set(["c","d"],\@a2);
    $c->unique_append(["a","b"],[1223]);
    $c->unique_append(["c","d"],[1223]);
    $c->unique_append(["c1"],[1223]);
    $c->unique_append(["c","e"],[1223]);
    
    
    compare("unique_append testing",join(':',$c->get(["a","b"])),join(':',(5,76,8,8,8,1223)));
    compare("unique_append testing",join(':',$c->get(["c","d"])),join(':',(2,6,89,0,89,0),(1223)));
    compare("unique_append testing",join(':',$c->get(["c","e"])),join(':',1223));

    $c->append(["a","b"],["fg"]);
    
    compare("append testing",join(':',$c->get(["a","b"])),join(':',(5,76,8,8,8,1223,"fg")));

    $c->remove(["a","b"],["fg"]);
    
    compare("remove testing",join(':',$c->get(["a","b"])),join(':',(5,76,8,8,8,1223)));

    @a1=(1,2,3,4,5);
    $c->set(["a1","b1","c1","d1"],\@a1);
    $c->set(["a1","b1","c1","d2"],\@a1);
    $c->set(["a2","b2","c2","d3"],\@a1);
    $c->set(["a1","sa"],[1]);

    my (@ref_arr);
    @ref_arr=$c->subpaths_at(["a1"]);
    compare (ref($c)." subpaths_at testing", &refarraypack(@ref_arr), '[b1:c1:d1][b1:c1:d2][sa]');

    @ref_arr=$c->paths_prefix(["a1"]);
    compare (ref($c)." paths_prefix testing", &refarraypack(@ref_arr), '[a1:b1:c1:d1][a1:b1:c1:d2][a1:sa]');

    if ($c->has_keys_at()) {
	@ref_arr=$c->keys_at(["a1"]);
	compare (ref($c)." keys_at testing", &arraypack(@ref_arr), 'sa:b1');
    }

    @ref_arr=$c->paths_all();
    compare (ref($c)." paths_all testing", &refarraypack(@ref_arr), 
	     '[a1:b1:c1:d1][a1:b1:c1:d2][a1:sa][a2:b2:c2:d3][a:b][c1][c:d][c:e]');

    if ($c->has_find()) {
	compare("find testing",join(':',$c->find(["a1","b1","c1",5])),join(':',("d1","d2")));
    }

    $c->clear_prefix(["a1"]);
    compare("clear_prefix testing",join(':',$c->get(["a1","b1","c1","d1"])),'');
    compare("clear_prefix testing",join(':',$c->get(["a1","b1","c1","d2"])),'');
    compare("clear_prefix testing",join(':',$c->get(["a2","b2","c2","d3"])),join(':',@a1));
    compare("clear_prefix testing",join(':',$c->get(["a1","sa"])),'');

    @ref_arr=$c->paths_all();
    @ref_arr=$c->paths_all();
    compare (ref($c)." paths_all before clone", &refarraypack(@ref_arr), '[a2:b2:c2:d3][a:b][c1][c:d][c:e]');

    my $clone=$c->clone();
    @ref_arr=$clone->paths_all();
    compare (ref($c)." paths_all after clone", &refarraypack(@ref_arr), '[a2:b2:c2:d3][a:b][c1][c:d][c:e]');
    compare("clone testing",join(':',$clone->get(["a2","b2","c2","d3"])),join(':',@a1));

    my @arrval=(1,2,3);
    $c->set(["memcorrupt", 0], \@arrval);
    push @arrval, 4;
    compare("memory copying",join(':',$c->get(["memcorrupt", 0])),join(':',(1,2,3)));

}

sub refarraypack {return '['.join('][',sort map { join(':',@$_)} @_).']'}
sub arraypack {return join(':',map {defined $_ ? $_ : 'undef' } @_)}
    
sub compare {
  $_[1]='' if !defined $_[1];
  $_[2]='' if !defined $_[2];
  diag ($_[0].": got {".$_[1]."} expected {".$_[2]."}");
  is ($_[1],$_[2],$_[0]);
}

sub assertTrue {
  $_[1]='' if !defined $_[1];
  #diag($_[0].": got {".$_[1]."}");
  ok($_[1],$_[0]);
}

sub assertFalse {
  $_[1]='' if !defined $_[1];
  #diag($_[0].": got {".$_[1]."}");
  ok(!$_[1],$_[0]);
}

